package org.apache.derby.impl.services.hotstandby;

import java.sql.Connection;
import java.sql.Driver;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.util.Properties;

import org.apache.derby.iapi.reference.Attribute;
import org.apache.derby.impl.jdbc.EmbedConnection;

/**
 * Simple helper class for an internal database connection
 * 
 * @author Egil Srensen
 *
 */

public class Database {

	protected String name;
	private Connection conn;
	private Driver driver;
	private String username;
	private String password;
	
	
	/**
	 * Initialize a database
	 * 
	 * @param dbName		The name of the database
	 * @param username		The database scheme/username for the database
	 * @param password		The password needed to connect to this database
	 * @throws SQLException
	 */
	public Database(String dbName, String username, String password) throws SQLException{
		this.username = username;
		this.password = password;
		name = dbName;
		driver = DriverManager.getDriver(Attribute.PROTOCOL);
	}
	
	
	/**
	 * Get the connection for this database
	 * 
	 * @return					A jdbc-connection to this database
	 * @throws SQLException
	 */
	public Connection getConnection() throws SQLException{
		if (conn == null)
			makeConnection(new Properties());
		return conn;
	}
	
	
	/**
	 * Make a connection to this database
	 */
	private void makeConnection(Properties p) throws SQLException{
		p.put(Attribute.USERNAME_ATTR, username);
		p.put(Attribute.PASSWORD_ATTR, password);
		conn = (EmbedConnection) driver.connect(Attribute.PROTOCOL + name, p);
	}
	
}
