package org.apache.derby.impl.services.net;

import java.io.Serializable;

/**
 * A class representing a network payload to be sent using the LogicalServer and 
 * LogicalClient. Contains a payload and a type-id. After a NetworkPayload has been 
 * created it is read-only to ensure safety and security in the network protocol
 * 
 * @author Egil Srensen
 *
 */

public class NetworkPayload implements Serializable{
	

	private static final long serialVersionUID = 3121376589155465699L;
	// The payload
	private Object payload;
	// The type of the payload
	private int type;
	
	// Constants
	public static final int TYPE_LOG = 0;
	public static final int TYPE_ACK = 1;
	public static final int TYPE_2PC_PREP = 2;
	public static final int TYPE_2PC_VOTE_COMMIT = 3;
	public static final int TYPE_2PC_VOTE_ABORT = 4;
	public static final int TYPE_2PC_DOCOMMIT = 5;
	public static final int TYPE_2PC_COMMIT_ACK = 6;
	public static final int TYPE_IMALIVE = 7;
	public static final int TYPE_MESSAGE_STRING = 8;
	public static final int TYPE_LSN_REQUEST = 9;
	public static final int TYPE_LSN_RESPONSE = 10;
	public static final int TYPE_ERROR = 11;
	public static final int TYPE_WAIT = 12;
	
	/**
	 * Constructor used to set the type and payload
	 * 
	 * @param type			The type of this payload. Must be one of the constants of this class
	 * @param payload		The payload. Can be any object
	 */
	public NetworkPayload(int type, Object payload){
		this.type = type;
		this.payload = payload;
	}
	
	/**
	 * Get the object payload
	 * 
	 * @return		The object contained in the payload
	 */
	public Object getPayload(){
		return payload;
	}
	
	
	/**
	 * Get the type of this NetworkPayload
	 * 
	 * @return		The type
	 */
	public int getType(){
		return type;
	}

}
