package org.apache.derby.impl.services.net.receiver;

import org.apache.derby.impl.services.hotstandby.HotStandbyController;
import org.apache.derby.impl.services.net.LogicalServer;
import org.apache.derby.impl.store.logical.log.LogicalLog;
/**
 * The LogReceiver is a simple process receiving log records and storing 
 * these on disk at regular intervals.
 * 
 * @author Egil Srensen
 *
 */
public class LogReceiver{
	// The active log running on this LogReceiver
	private LogicalLog log;
	// The number of logs received so far
	private int logsReceived;
	
	private LogicalServer transport;
	private static LogReceiver instance;
	
	public static LogReceiver getInstance(){
		if (HotStandbyController.getInstance().isInHotStandbyMode()){
		
			if (instance == null)
				instance = new LogReceiver();
			return instance;
		}
		else
			return null;
	}
	
	
	/**
	 * Constructor that initiates a new LogReceiver
	 *
	 */
	private LogReceiver(){
		logsReceived = 0;
		transport = LogicalServer.getInstance();
		log = LogicalLog.readFromDisk();
	}
	
	
	/**
	 * Receive an incoming log and hand it to HotStandbyController
	 * 
	 * @param newLog		The received log
	 */
	public synchronized void receive(LogicalLog newLog){
		
//		System.out.println("[receive] Received log: [" + newLog.getFirst().getLsn() + " - " + newLog.getLast().getLsn() + "]");
		
		// Add the incoming log to the active log
//		log.addElements(newLog);
		
		// Clean up resources
		
//		System.out.println("[receive] I have received " + ++logsReceived + " logs by now.");
//		System.out.println("[status] Log size is now: " + log.size());
		
//		System.out.println("[status] Free memory: " + Runtime.getRuntime().freeMemory());

		
		
		
//		System.out.println("Handing log to HotStandbyController");
		// Give the log to the HotStandbyController
		HotStandbyController.getInstance().handleIncomingLogs(newLog);
		// Flush the log to disk when finished
//		log.flushLogToDisk();
		newLog = null;
	}
	
	
	/**
	 * Get the active log from the LogReceiver
	 * 
	 * @return		The active log
	 */
	public LogicalLog getLog(){
		return log;
	}
	
	
	/**
	 * Get the next lsn from the active log
	 * 
	 * @return		The next lsn
	 */
	
	public int getNextLsn(){
		if (log.size() > 0)
			return log.getLast().getLsn() + 1;
		else
			return 1;
	}
	
	
	

	
}
