package org.apache.derby.impl.store.logical;

import java.util.ArrayList;

import org.apache.derby.iapi.store.logical.log.LogicalLoggable;
import org.apache.derby.impl.store.logical.operations.LogicalOperation;

/**
 * 
 * Used by the Hot Standby Controller to create logical transactions
 * 
 * @author egil
 *
 */
public class LogicalTransaction {

	public static final int STATUS_COMMITTED = 0;
	public static final int STATUS_ABORTED = 1;
	public static final int RUNNING = 2;
	
	private int transId;
	private ArrayList operations;
	private int status;
	
	public LogicalTransaction(int transId){
		this.transId = transId;
		operations = new ArrayList();
		status = RUNNING;
	}
	
	public void addOperation(LogicalLoggable op){
		if (status != STATUS_ABORTED)
			operations.add(op);
	}
	
	public ArrayList getOperations(){
		return operations;
	}
	
	public int getTransId(){
		return transId;
	}
	
	public int getStatus(){
		return status;
	}
	
	public void setStatus(int status){
		switch (status){
		case RUNNING:
			status = RUNNING;
			break;
		case STATUS_COMMITTED:
			status = STATUS_COMMITTED;
			break;
		case STATUS_ABORTED:
			status = STATUS_ABORTED;
			break;
		default:
			break;
		}
	}
	
	public String toString(){
		String returned = "Transaction " + transId + "\n\n";
		
		for (int i = 0; i < operations.size(); i++){
			returned += operations.get(i) + "\n";
		}
		
		
		
		return returned;
	}
	
}
