package org.apache.derby.impl.store.logical.log;

import java.io.Serializable;

import org.apache.derby.iapi.store.logical.log.LogicalLoggable;
import org.apache.derby.impl.store.logical.operations.LogicalOperation;

import com.sun.rsasign.o;

/**
 * The implementation of a Logical Log Record
 * 
 * @author Egil Srensen
 * @see		org.apache.derby.impl.store.logical.log.LogicalLogger
 */
public class LogicalLogRecord implements Serializable{


	// The lsn of the log record
	private int lsn;
	// The previous lsn for this transaction
	private int prevTransLsn;
	// The transaction id of the owner transaction
	private int transid;
	// The operation this log record corresponds to
	private LogicalLoggable operation;
	
	/**
	 * Constructor for creating a logical log record
	 * 
	 * @param lsn				the lsn of the log record
	 * @param prevTransLsn		the previous lsn for this transaction
	 * @param transid			the transaction id of the owner transaction
	 * @param operation			the operation this log record corresponds to
	 */
	public LogicalLogRecord(int lsn, int prevTransLsn, int transid, LogicalLoggable operation){
		this.lsn = lsn;
		this.prevTransLsn = prevTransLsn;
		this.transid = transid;
		
		this.operation = operation;
		
	}
	
	/**
	 * Get the lsn for this log record
	 * 
	 * @return		the lsn
	 */
	public int getLsn(){
		return lsn;
	}
	
	/**
	 * Get prevTransLsn
	 * 
	 * @return		prevTransLsn
	 */
	public int getPrevTransLsn(){
		return prevTransLsn;
	}
	
	/**
	 * Get transId
	 * 
	 * @return		transId
	 */
	public int getTransId(){
		return transid;
	}
	
	/**
	 * toString, created for debugging purposes, shows the 
	 * contents of the log record
	 */
	public String toString() {
		return "Logical log record #" + lsn + "\n" +
				"Transaction id: " + transid + "\n" +
				"Previous LSN for this Transaction: " + prevTransLsn + "\n" +
				"Operation: " + 
				operation;
	}
	
	
	public LogicalLoggable getOperation(){
		return operation;
	}
	
	
}
