package org.apache.derby.impl.store.logical.operations;

import org.apache.derby.iapi.store.logical.log.LogicalImage;
import org.apache.derby.iapi.store.logical.log.LogicalLoggable;
import org.apache.derby.iapi.types.DataValueDescriptor;
/**
 * A Logical Commit Operation, contains no undo or redo image
 * 
 * @author Egil Srensen
 * 
 * @see org.apache.derby.impl.store.logical.operations.LogicalImage
 *
 */
public class LogicalCommitOperation extends LogicalOperation {


	private static final long serialVersionUID = 2404910542679338238L;

	public LogicalCommitOperation(LogicalImage beforeImage,
			LogicalImage afterImage, DataValueDescriptor oldKey,
			DataValueDescriptor newKey, String tableName) {
		super(beforeImage, afterImage, oldKey, newKey, tableName);
	}
	
	public boolean isUndoable() {
		return false;
	}
	
	public boolean isRedoable() {
		return false;
	}
	
	public LogicalImage getUndoImage() {
		return null;
	}
	
	public LogicalImage getRedoImage() {
		return null;
	}
	
	public int group() {
		return LogicalLoggable.COMMIT_OPERATION;
	}
	
	public String toString(){
		return "COMMIT";
	}

}
