package org.apache.derby.impl.store.logical.operations;

import java.util.Enumeration;
import java.util.Hashtable;

import org.apache.derby.iapi.store.logical.log.LogicalImage;
import org.apache.derby.iapi.store.logical.log.LogicalLoggable;
import org.apache.derby.iapi.types.DataValueDescriptor;
/**
 * A Logical Delete Operation, contains undo image only
 * 
 * @author Egil Srensen
 * 
 * @see org.apache.derby.impl.store.logical.operations.LogicalImage
 *
 */
public class LogicalDeleteOperation extends LogicalOperation {

	private static final long serialVersionUID = 9003105796974012615L;

	public LogicalDeleteOperation(LogicalImage beforeImage, LogicalImage afterImage, DataValueDescriptor oldKey, DataValueDescriptor newKey, String tableName) {
		super(beforeImage, afterImage, oldKey, newKey, tableName);
		
		createRedo();
		createUndo();
	}
	
	/**
	 * Create a redo-statement for this operation
	 *
	 */
	private void createRedo(){
		redoStmt = "DELETE FROM " + tableName + " WHERE " + beforeImage.getKeyColumnName() + " = " + beforeImage.getKey();
	}
	
	/**
	 * Create an undo-statement for this operation
	 *
	 */
	private void createUndo(){
		undoStmt = "INSERT INTO " + tableName + " (";
		
		String columns = "";
		String values = "";
		
		Hashtable contents = beforeImage.getContents();
		
		Enumeration keys = contents.keys();
		
		while (keys.hasMoreElements()){
			String key = (String) keys.nextElement();
			String value = (String) contents.get(key);
			columns += key;
			values += value;
			if (keys.hasMoreElements()){
				columns += ", ";
				values += ", ";
			}
		}
		
		undoStmt += columns + ") VALUES (" + values;
		
		undoStmt += ")";
	}
	

	public LogicalImage getRedoImage() {
		return null;
	}
	
	public int group() {
		return LogicalLoggable.DELETE_OPERATION;
	}
	
	
	public String toString(){
		String returned = "DELETE FROM " + tableName + " WHERE " + beforeImage.getKeyColumnName() + " = " + oldKey;
		
		
		return returned;
	}
}
