package org.apache.derby.impl.store.logical.operations;

import java.util.Enumeration;
import java.util.Hashtable;

import org.apache.derby.iapi.services.io.StoredFormatIds;
import org.apache.derby.iapi.sql.dictionary.ColumnDescriptor;
import org.apache.derby.iapi.types.DataTypeDescriptor;
import org.apache.derby.iapi.types.DataValueDescriptor;


/**
 * An implementation of a logical image. Can be either a before- or after image
 * 
 * @author Egil Srensen
 *
 */
public class LogicalImage implements
		org.apache.derby.iapi.store.logical.log.LogicalImage {
	
	// Automatically generated serial version id
	private static final long serialVersionUID = 4755778266295898312L;
	// The contents of the logical image, contains (name of row ==> contents)
	private Hashtable contents;
	// The type of this logical image
	private int type;
	// The name of the key of this logical image
	private String key;
	// The name of the key column of this logical image
	private String keyColumnName;
	
	/**
	 * Constructor that creates a logicalimage from the internal types 
	 * columndescriptor and datavaluedescriptor so that they can be 
	 * serialized and sent through a outputstream
	 * 
	 * @param columns		The columns of the affected table
	 * @param values		The values of the affected row
	 * @param key			The key of the affected table
	 * @param type			The type of this image, can either be before- or after image
	 * 
	 * @see			org.apache.derby.impl.store.logical.log.LogicalLogger
	 */
	public LogicalImage(ColumnDescriptor[] columns, DataValueDescriptor[] values, int keyColumn, DataValueDescriptor key, int type){
		contents = new Hashtable();
		
		// If there are any values add these to the contents
		if (values != null && columns != null){
			for (int i = 0; i < columns.length; i++){
				String value = values[i].toString();

				
				if (! DataTypeDescriptor.isNumericType(columns[i].getType().getJDBCTypeId()))
					value = "'" + value.trim() + "'";
				
				
				contents.put(columns[i].getColumnName(), value);

			}
		}
		
		// Add the key if it exists
		if (key == null){
			this.key = "";
			keyColumnName = "";
		}
		else{
			this.keyColumnName = columns[keyColumn].getColumnName();
			if (! DataTypeDescriptor.isNumericType(columns[keyColumn].getType().getJDBCTypeId()))
				this.key = "'" + key.toString() + "'";
			else
				this.key = key.toString();
		}
		// Set the type
		this.type = type;
	}
	
	
	

	public Hashtable getContents() {
		return contents;
	}

	public String getKey() {
		return key;
	}

	public int getType() {
		return type;
	}

	public int size() {
			return contents.size();
		
	}
	
	public String getKeyColumnName(){
		return keyColumnName;
	}
	

	
	/**
	 * Returns a String representation of the image, used for debugging purposes
	 */
	
	public String toString(){
		String returned = "";
		
		Enumeration keys = contents.keys();
		
		while (keys.hasMoreElements()){
			String key = (String) keys.nextElement();
			String content = (String) contents.get(key);
			returned += "[" + key + "] " + content + "\n";
		}

		
		return returned;
	}

}
