package org.apache.derby.impl.store.logical.operations;


import java.util.Enumeration;
import java.util.Hashtable;

import org.apache.derby.iapi.store.logical.log.LogicalImage;
import org.apache.derby.iapi.store.logical.log.LogicalLoggable;
import org.apache.derby.iapi.types.DataValueDescriptor;

/**
 * A Logical Insert Operation, contains redo image only
 * 
 * @author Egil Srensen
 * 
 * @see org.apache.derby.impl.store.logical.operations.LogicalImage
 *
 */
public class LogicalInsertOperation extends LogicalOperation{
	
	
	private static final long serialVersionUID = -2756798443496783420L;

	public LogicalInsertOperation(LogicalImage beforeImage, LogicalImage afterImage, DataValueDescriptor oldKey, DataValueDescriptor newKey, String tableName) {
		super(beforeImage, afterImage, oldKey, newKey, tableName);
		
		createUndo();
		createRedo();
	}
	
	/**
	 * Create an undo-statement for this operation
	 *
	 */
	private void createUndo(){
		undoStmt = "DELETE FROM " + tableName + " WHERE " + afterImage.getKeyColumnName() + " = " + afterImage.getKey();
	}
	
	/**
	 * Create a redo-statement for this operation
	 *
	 */
	private void createRedo(){
		redoStmt = "INSERT INTO " + tableName + " (";
		
		String columns = "";
		String values = "";
		
		Hashtable contents = afterImage.getContents();
		
		Enumeration keys = contents.keys();
		
		while (keys.hasMoreElements()){
			String key = (String) keys.nextElement();
			String value = (String) contents.get(key);
			columns += key;
			values += value;
			if (keys.hasMoreElements()){
				columns += ", ";
				values += ", ";
			}
		}
		
		redoStmt += columns + ") VALUES (" + values;
		
		redoStmt += ")";
	}




	public LogicalImage getUndoImage() {
		return null;
	}
	
	public int group() {
		return LogicalLoggable.INSERT_OPERATION;
	}
	
	


}
