package org.apache.derby.impl.store.logical.operations;

import java.util.Enumeration;
import java.util.Hashtable;

import org.apache.derby.iapi.store.logical.log.LogicalImage;
import org.apache.derby.iapi.store.logical.log.LogicalLoggable;
import org.apache.derby.iapi.types.DataValueDescriptor;
/**
 * A Logical Undo Operation, contains both redo and undo images
 * 
 * @author Egil Srensen
 * 
 * @see org.apache.derby.impl.store.logical.operations.LogicalImage
 *
 */
public class LogicalUpdateOperation extends LogicalOperation{

	private static final long serialVersionUID = 7376047523027256627L;


	public LogicalUpdateOperation(LogicalImage beforeImage, LogicalImage afterImage, DataValueDescriptor oldKey, DataValueDescriptor newKey, String tableName) {
		super(beforeImage, afterImage, oldKey, newKey, tableName);
		
		createUndo();
		createRedo();
	}
	
	
	/**
	 * Create an undo-statement for this operation
	 *
	 */
	private void createUndo(){
		undoStmt = "UPDATE " + tableName + " SET ";
		
		Hashtable contents = beforeImage.getContents();
		Enumeration keys = contents.keys();
		
		while (keys.hasMoreElements()){
			String key = (String) keys.nextElement();
			String value = (String) contents.get(key);
			undoStmt += key + "=" + value;
			if (keys.hasMoreElements())
				undoStmt += ", ";
		}
		
		
		undoStmt += " WHERE " + afterImage.getKeyColumnName() + " = " + newKey;
	}
	
	
	/**
	 * Create a redo-statement for this operation
	 *
	 */
	private void createRedo(){
		redoStmt = "UPDATE " + tableName + " SET ";
		
		Hashtable contents = afterImage.getContents();
		Enumeration keys = contents.keys();
		
		while (keys.hasMoreElements()){
			String key = (String) keys.nextElement();
			String value = (String) contents.get(key);
			redoStmt += key + "=" + value;
			if (keys.hasMoreElements())
				redoStmt += ", ";
		}
		
		
		redoStmt += " WHERE " + beforeImage.getKeyColumnName() + " = " + oldKey;
		
	}
	
	

	
	
	public int group() {
		return LogicalLoggable.UPDATE_OPERATION;
	}
	

	
}